Script.stereoEnabled = true --bool "Enable Stereo"
Script.stereoOffset = 0.02 --float "Eye-Separation"
Script.stereoAngularOffset = 0.93 --float "Angular Offset"
Script.nearplane = 0.01 --float "Nearplane"
Script.farplane = 1000 --float "Farplane"
Script.camera = nil --entity "Camera"
Script.shaderpath = "AddOns/Interlaced Stereo 3D/stereo_interlaced.shader" --string "Shader-Path" 

function Script:Start()
	if(self.camera == nil) then
		self.camera = self.entity
	end
	local context = Context:GetCurrent()
	self.leftEye = Buffer:Create(context:GetWidth() + 1, context:GetHeight() + 1, 1, 1, 0)
	self.camera:SetRange(self.nearplane, self.farplane)
	
	self.stereoShader = Shader:Load(self.shaderpath)
end

function Script:CheckStereoKeys()
	local window = Context:GetCurrent():GetWindow()
	if (window:KeyDown(Key.NumPadAddition)) then
		self.stereoOffset = self.stereoOffset + 0.01
	end
	if (window:KeyDown(Key.NumPadSubtract)) then
		self.stereoOffset = self.stereoOffset - 0.01
	end
	if (window:KeyDown(Key.NumPadMultiply)) then
		self.stereoAngularOffset = self.stereoAngularOffset + 0.1
	end
	if (window:KeyDown(Key.NumPadDivide)) then
		self.stereoAngularOffset = self.stereoAngularOffset - 0.1
	end
	if (window:KeyHit(Key.Tab)) then
		self.stereoEnabled = not(self.stereoEnabled)
	end
	if (window:KeyHit(Key.F1)) then
		self.stereoShader:Reload()
	end
end

function Script:RenderStereo()
	local context = Context:GetCurrent()
	local world = World:GetCurrent()
	if (self.stereoEnabled) then
		context:Clear()
		self.leftEye:Clear()
		
		context:Disable()
		offhalf = self.stereoOffset / 2.0
		anghalf = self.stereoAngularOffset / 2.0
		
		--Render left eye interlaced to temporary buffer
		self.leftEye:Enable()
		self.camera:AddPostEffect(self.stereoShader.path)
		self.camera:Move(-offhalf, 0, 0, false)
		self.camera:Turn(0, -anghalf,0, false)
		
		world:Render()
		
		self.camera:Turn(0, anghalf, 0, false)
		self.camera:Move(offhalf, 0, 0, false)
		--self.camera.posteffects:pop_back() TODO
		self.camera:ClearPostEffects()
		self.leftEye:Disable()
		
		--Move Camera and render right eye completely to backbuffer
		context:Enable()
		self.camera:Move(offhalf, 0, 0, false)
		self.camera:Turn(0, anghalf, 0, false)
		
		world:Render()
		
		self.camera:Turn(0, -anghalf, 0, false)
		self.camera:Move(-offhalf, 0, 0, false)
		
		--Draw the previously rendered left eye over the right eye
		local blendModeOld = context:GetBlendMode()
		context:SetBlendMode(Blend.Alpha)
		context:DrawImage(self.leftEye:GetColorTexture(), 0, 0, context:GetWidth()+1, context:GetHeight()+1)
		context:SetBlendMode(blendModeOld)
	else
		world:Render()
	end
end

function Script:DrawStereoInfo(posx, posy)
	local context = Context:GetCurrent()
	local blendModeOld = context:GetBlendMode();
	context:SetBlendMode(Blend.Alpha);
	local s = "Offset" .. self.stereoOffset
	context:DrawText(s, posx, posy);

	s = "Angular Offset: " .. self.stereoAngularOffset
	context:DrawText(s, posx, posy + 15);

	if (self.StereoEnabled) then
		context:DrawText("3D", posx, posy + 30);
	end
	context:SetBlendMode(blendModeOld);
end

function Script:UpdateWorld()
	self:CheckStereoKeys()
	self:RenderStereo()
	self:DrawStereoInfo(10,150)
end